/**
IBM SOFTWARE DISCLAIMER 

Java array package (draft 0.2). Copyright (1998), International Business 
Machines Corporation.

Permission to use, copy, modify and distribute this software for any
noncommercial purpose and without fee is hereby granted, provided that
this copyright and permission notice appear on all copies of the
software. The name of the IBM Corporation may not be used in any
advertising or publicity pertaining to the use of the software. IBM
makes no warranty or representations about the suitability of the
software for any purpose.  It is provided "AS IS" without any express
or implied warranty, including the implied warranties of
merchantability, fitness for a particular purpose and non-infringement.
IBM shall not be liable for any direct, indirect, special or
consequential damages resulting from the loss of use, data or projects,
whether in an action of contract or tort, arising out of or in
connection with the use or performance of this software.
*/

package array;

/**
   Index = represents an index set to arrays
   <P>
   Ranges provide the support to extract and operate on regular 
   array sections. Indexes provide similar support for irregular
   sections.
   It is not possible to define an array section using Indexes, but
   one can use them in gather/scatter-like operations.
   <P>
   The only restriction is that elements in an Index must be nonnegative.
   The same value can appear multiple times in an Index.
   Indexes can be empty.
*/

public final class Index {

    protected int[] elements;
    protected int theMin = Integer.MAX_VALUE;
    protected int theMax = Integer.MIN_VALUE;

    /**
       Create an Index from a vector of ints.
       @param list	vector of int with list of elements
       @exception InvalidIndexException elements must be nonnegative
    */
    public Index(int[] list) throws InvalidIndexException {
	
	if (list == null) {
	    elements = new int[0];
	} else {
	    elements = new int[list.length];
	    for (int i=0; i<list.length; i++) {
		if (list[i] < 0) throw new InvalidIndexException();
		elements[i] = list[i];
		if (elements[i] > theMax) theMax = elements[i];
		if (elements[i] < theMin) theMin = elements[i];
	    }
	}
    }
	
    /**
       Create and Index from a Range. Note that a Range cannot have
       negative elements.
       @param range	Range describing elements
    */
    public Index(Range range) {

	elements = new int[range.length()];
	try {
	    for (int i=0; i<elements.length; i++) {
		elements[i] = range.element(i);
		if (elements[i] > theMax) theMax = elements[i];
		if (elements[i] < theMin) theMin = elements[i];
	    }
	} catch (InvalidRangeElementException e) {
	    // This error should not occur.
	    elements = new int[0];
	    theMin = Integer.MAX_VALUE;
	    theMax = Integer.MIN_VALUE;
	}
    }

    /**
       Return the number of elements in an Index.
    */
    public int length() {

	return elements.length;
    }

    /**
       Return the i-th element of an Index.
       @param i		index of the element
       @exception InvalidIndexElementException	0 <= i < Index.length()
    */
    public int element(int i) throws InvalidIndexElementException {
	
	if (i < 0) throw new InvalidIndexElementException();
	if (i >= elements.length) throw new InvalidIndexElementException();
	return elements[i];
    }

    /**
       Return the i-th element of an Index, no check.
       @param i		index of the element
    */
    protected int elementf(int i) {

	return elements[i];
    }

    /**
       Return the smallest element in an Index.
    */
    public int min() {
	
	return theMin;
    }

    /**
       Return the largest element in an Index.
    */
    public int max() {

	return theMax;
    }

    /**
       Return an int[] with the elements of this Index.
    */
    public int[] toJava() {

	int n = length();
	int[] result = new int[n];

	for (int i=0; i<n; i++) {
	    result[i] = elementf(i);
	}

	return result;
    }
}



