/**
IBM SOFTWARE DISCLAIMER 

Java array package (draft 0.2). Copyright (1998), International Business 
Machines Corporation.

Permission to use, copy, modify and distribute this software for any
noncommercial purpose and without fee is hereby granted, provided that
this copyright and permission notice appear on all copies of the
software. The name of the IBM Corporation may not be used in any
advertising or publicity pertaining to the use of the software. IBM
makes no warranty or representations about the suitability of the
software for any purpose.  It is provided "AS IS" without any express
or implied warranty, including the implied warranties of
merchantability, fitness for a particular purpose and non-infringement.
IBM shall not be liable for any direct, indirect, special or
consequential damages resulting from the loss of use, data or projects,
whether in an action of contract or tort, arising out of or in
connection with the use or performance of this software.
*/

package array;

/**
   doubleArray0D = 0-dimensional array of doubles (scalar)
   <P>
   The doubleArray0D concrete class implements 0-dimensional arrays
   of doubles. 0-dimensional arrays are scalars. Although they can
   be instantiated directly through constructors, 
   0-dimensional arrays are typically 
   generated by extracting single-element sections from 
   higher-dimensional arrays.
*/
   
public final class doubleArray0D extends doubleArray {

    private double[] data;
    private int a0;

    /**
       Create a 0-dimensional array with a 0-valued element.
    */
    public doubleArray0D() {
	
	data = new double[1];
	data[0] = 0.0;
	a0 = 0;
    }

    /**
       Create a 0-dimensional array. The value of the element
       is specified by a parameter.
       @param a	value of element
    */
    public doubleArray0D(double a) {

	data = new double[1];
	data[0] = a;
	a0 = 0;
    }

    /**
       Create a 0-dimensional array as a section of another array.
       Internal use within the package only.
       A 0-dimensional array section can be empty (have 0 elements).
    */
    protected doubleArray0D(double[] d, int a) {

	data = d;
	if (a >= d.length) throw new ArrayIndexOutOfBoundsException();
	a0 = a;
    }

    /**
       Return the rank of the 0-dimensional array, i.e. 0.
    */
    public int rank() {

	return 0;
    }

    /**
       Return the number of elements in the array. This number is
       0 if the array is empty, 1 otherwise.
    */
    public int size() {

	if (a0 >= 0) return 1;
	else return 0;
    }

    /**
       A 0-dimensional array does not have any axis. Therefore, 
       any invocation of size(i) throws an
       InvalidArrayAxisException exception.
       @param i	array axis (0 <= i < rank())
       @exception InvalidArrayAxisException always thrown
    */
    public int size(int i) throws InvalidArrayAxisException {

	throw new InvalidArrayAxisException();
    }

    /**
       Returns the shape of a 0-dimensional array. This is an
       int[] of length 0.
    */
    public int[] shape() {

	return new int[0];
    }

    /**
       Return the value of an element of the array.
       @param index	index[i] = index along i-th axis, 0 <= index[i] < size(i)
       @exception InvalidArrayIndexException index[] must be of length 0
    */
    public double get(int[] index)
	throws InvalidArrayIndexException, ArrayIndexOutOfBoundsException {

	if (index.length != 0) throw new InvalidArrayIndexException();
	return get();
    }

    /**
       Set the value of an element of the array.
       @param index	index[i] = index along i-th axis, 0 <= index[i] < size(i)
       @param d		value to which element is set
       @exception InvalidArrayIndexException index[] must be of length 0
    */
    public void set(int [] index, double d) 
	throws InvalidArrayIndexException, ArrayIndexOutOfBoundsException {

	if (index.length != 0) throw new InvalidArrayIndexException();
	set(d);
    }

    /**
       Returns the value of the element of the 0-dimensional array, 
       if any.
    */
    public double get() {
	
	if (a0 < 0) throw new ArrayIndexOutOfBoundsException();
	return data[a0];
    }

    /**
       Sets the value of the element of the 0-dimensional array, if any.
       @param d		value for the element
    */
    public void set(double d) {

	if (a0 < 0) throw new ArrayIndexOutOfBoundsException();
	data[a0] = d;
    }

    /**
       Assign a scalar value to all elements of the array.
       @param d		value to be assigned
    */
    public void assign(double d) {

	set(d);
    }

    /**
       Assign the values from another zero-dimensional array to this array.
       @param a		array with source value
       @exception NonconformingArrayException arrays must be of same shape
    */
    public void assign(doubleArray0D a)
	throws NonconformingArrayException {
	
	int n = size();
	if (n != a.size()) throw new NonconformingArrayException();

	set(a.get());
    }
}
