/**
IBM SOFTWARE DISCLAIMER 

Java array package (draft 0.2). Copyright (1998), International Business 
Machines Corporation.

Permission to use, copy, modify and distribute this software for any
noncommercial purpose and without fee is hereby granted, provided that
this copyright and permission notice appear on all copies of the
software. The name of the IBM Corporation may not be used in any
advertising or publicity pertaining to the use of the software. IBM
makes no warranty or representations about the suitability of the
software for any purpose.  It is provided "AS IS" without any express
or implied warranty, including the implied warranties of
merchantability, fitness for a particular purpose and non-infringement.
IBM shall not be liable for any direct, indirect, special or
consequential damages resulting from the loss of use, data or projects,
whether in an action of contract or tort, arising out of or in
connection with the use or performance of this software.
*/

import array.*;

public class TestBlas {


    public static void print(doubleArray2D C) {

	try {
	    int m = C.size(0);
	    int p = C.size(1);

	    for (int i=0; i<m; i++) {
		if (i > 0) System.out.println();
		if (p > 0) System.out.print(C.get(i,0));
		for (int j=1; j<p; j++) {
		    System.out.print(" "+C.get(i,j));
		}
	    }
	    System.out.println();
	} catch (InvalidArrayAxisException e) {
	    System.out.println("Unexpected error in print, terminating ...");
	    System.exit(-1);
	}
    }


    public static boolean valid(doubleArray2D C, int n) {
	
	/*
	 * Check validity of results. Element C(i,j) should be 
	 * equal to i*n*(n-1)/2 (within an epsilon).
	 */
	double eps = 1e-09;

	try {
	    int m = C.size(0);
	    int p = C.size(1);

	    for (int i=0; i<m; i++) {
		for (int j=0; j<p; j++) {
		    double correct = ((double)i*(double)n*(double)(n-1)/2.0);
		    double error = Math.abs(C.get(i,j)-correct);
		    if (error > eps) {
			return false;
		    }
		}
	    }
	} catch (InvalidArrayAxisException e) {
	    System.out.println("Unexpected error in valid, terminating ...");
	    System.exit(-1);
	}

	return true;
    }
    
    public static void main(String[] args) 
	throws InvalidArrayShapeException, InvalidRangeException,
	       NonconformingArrayException, InvalidArrayAxisException {

	int m = 4;
	int n = 4;
	int p = 4;
	boolean print = false;

	/*
	 * Parse command line arguments.
	 */
	try{
	    
	    for(int arg=0; arg<args.length; arg++) {
		if (args[arg].equals("-m")) 
		    m = Integer.parseInt(args[++arg]);
		else if (args[arg].equals("-n"))
		    n = Integer.parseInt(args[++arg]);
		else if (args[arg].equals("-p"))
		    p = Integer.parseInt(args[++arg]);
		else if (args[arg].equals("-print"))
		    print = true;
		else if (args[arg].equals("-h"))
		    throw new Exception();
		else if (args[arg].equals("-help"))
		    throw new Exception();
		else throw new Exception();
	    }
	} catch (Exception e) {
	    System.out.println("usage: MatMul [-m <m>] [-n <n>] [-p <p>] [-print]");
	    return;
	}

	System.out.println("matrix multiply: "+m+" x "+n+" x "+p);

	/*
	 * Initialize the matrices A(mxn) and B(nxp) so that
	 * row i of each has all elements equal to i.
	 */
	doubleArray2D A = new doubleArray2D(m,n);
	doubleArray2D B = new doubleArray2D(n,p);
	for (int i=0; i<m; i++) {
	    A.section(i,new Range(0,n-1)).assign(i);
	}
	for (int i=0; i<n; i++) {
	    B.section(i,new Range(0,p-1)).assign(i);
	}
	doubleArray2D C = new doubleArray2D(m,p);

	/*
	 * Use variable etime to measure elapsed time, mflops to
	 * compute Mflops.
	 */
	double etime = 0;
	double mflops = 0;

	/*
	 * Perform the matrix multiplication of A and B, yielding C.
	 * Use Blas.dgemm.
	 */
	etime = System.currentTimeMillis();
	Blas.dgemm(Blas.NoTranspose,Blas.NoTranspose,1.0,A,B,0.0,C);
	etime = System.currentTimeMillis() - etime;

	/*
	 * Check validity of result.
	 */
	if (!valid(C,n)) {
	    System.out.println("error in result of Blas.dgemm");
	}

	/*
	 * If print flag is set, print the result.
	 */
	if (print) {
	    System.out.println("C matrix from dgemm:");
	    print(C);
	}

	/*
	 * Print performance.
	 */
	mflops = 1.0e-3*(2.0*m*n*p)/etime;
	System.out.println("mflops from dgemm: "+mflops);

	/*
	 * Perform the matrix multiplication of A and B, yielding C.
	 * Use Blas.dgemv.
	 */
	etime = System.currentTimeMillis();
	for (int j=0; j<p; j++) {
	    Blas.dgemv(Blas.NoTranspose,1.0,A,B.section(B.range(0),j),
		       0.0,C.section(C.range(0),j));
	}
	etime = System.currentTimeMillis() - etime;

	/*
	 * Check validity of result.
	 */
	if (!valid(C,n)) {
	    System.out.println("error in result of Blas.dgemv");
	}

	/*
	 * If print flag is set, print the result.
	 */
	if (print) {
	    System.out.println("C matrix from dgemv:");
	    print(C);
	}

	/*
	 * Print performance.
	 */
	mflops = 1.0e-3*(2.0*m*n*p)/etime;
	System.out.println("mflops from dgemv: "+mflops);

	/*
	 * Perform the matrix multiplication of A and B, yielding C.
	 * Use Blas.ddot.
	 */
	etime = System.currentTimeMillis();
	for (int i=0; i<m; i++) {
	    for (int j=0; j<p; j++) {
		C.set(i,j,Blas.ddot(A.section(i,B.range(1)),
				    B.section(B.range(0),j)));
	    }
	}
	etime = System.currentTimeMillis() - etime;

	/*
	 * Check validity of result.
	 */
	if (!valid(C,n)) {
	    System.out.println("error in result of Blas.ddot");
	}

	/*
	 * If print flag is set, print the result.
	 */
	if (print) {
	    System.out.println("C matrix from ddot:");
	    print(C);
	}

	/*
	 * Print performance.
	 */
	mflops = 1.0e-3*(2.0*m*n*p)/etime;
	System.out.println("mflops from ddot: "+mflops);
    }
}
